<?php

namespace Drupal\role_toggle;

use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;
use Drupal\user\RoleInterface;

/**
 * Class RoleToggle
 *
 * @internal
 * @package Drupal\role_toggle
 */
class RoleToggle {

  public static function hookRoleDelete(RoleInterface $role) {
    $permissionIdToToggle = self::permissionIdToToggle($role);
    foreach (user_roles() as $role) {
      if ($role->hasPermission($permissionIdToToggle)) {
        $role->revokePermission($permissionIdToToggle);
        $role->save();
      }
    }
  }

  public static function permissions() {
    $perms = array();
    foreach (self::configurableRoles() as $role) {
      $perms[self::permissionIdToToggle($role)] = array(
        'title' => self::permissionLabel($role),
        'description' => t('Users with this permission have the right to toggle said role. Note that permissions from authenticated and admin role are ignored, as both lead to insane results.'),
      );
    }
    return $perms;
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return bool
   */
  public static function userAccess($role) {
    return \Drupal::currentUser()->hasPermission(self::permissionIdToToggle($role));
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return string
   */
  public static function permissionIdToToggle($role) {
    return 'role_toggle:' . $role->id();
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   */
  public static function permissionLabel($role) {
    return t('Toggle role %role', array('%role' => $role->label()));
  }

  /**
   * Get all applicable user roles.
   *
   * @return \Drupal\user\RoleInterface[]
   */
  public static function configurableRoles() {
    $roles = user_roles();
    unset($roles[RoleInterface::ANONYMOUS_ID]);
    unset($roles[RoleInterface::AUTHENTICATED_ID]);
    return $roles;
  }

  /**
   * @return \Drupal\user\RoleInterface[]
   */
  public static function togglableRoles() {
    return array_filter(self::configurableRoles(), function(RoleInterface $role) {
      return self::canToggle($role);
    });
  }

  /**
   * Check if a user can toggle a role.
   *
   * @param \Drupal\user\RoleInterface $role
   * @return bool
   */
  public static function canToggle($role, AccountInterface $account = NULL) {
    if (!$account) {
      $account = \Drupal::currentUser();
    }
    return self::accountHasExplicitPermission($account, self::permissionIdToToggle($role));
  }

  /**
   * Check if a user has permission, but ignore isAdmin() and auth perms.
   *
   * Having auth permissions to toggle a role is insane, so ignore that.
   * The admin role otoh leads to people being able to toggle away all roles
   * which is insane too.
   *
   * @see \Drupal\user\RoleStorage::isPermissionInRoles
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   * @param string $permissionId
   *
   * @return bool
   */
  protected static function accountHasExplicitPermission(AccountInterface $account, $permissionId) {
    $has_permission = FALSE;
    foreach ($account->getRoles(TRUE) as $roleId) {
      $role = Role::load($roleId);
      if (!$role->isAdmin() && $role->hasPermission($permissionId)) {
        $has_permission = TRUE;
        break;
      }
    }
    return $has_permission;
  }

  /**
   * @param \Drupal\Core\Session\AccountInterface|null $account
   * @return bool
   */
  public static function canToggleAny(AccountInterface $account = NULL) {
    if (!$account) {
      $account = \Drupal::currentUser();
    }
    foreach (self::configurableRoles() as $role) {
      if (self::canToggle($role, $account)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return bool
   */
  public static function isEnabledRole($role) {
    return in_array($role->id(), \Drupal::currentUser()->getRoles());
  }

  public static function indicatorClass($enabled) {
    return "role-toggle-role-" . ($enabled ? 'active' : 'inactive');
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   * @return bool
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public static function toggleRole($role) {
    $rid = $role->id();
    $currentUser = User::load(\Drupal::currentUser()->id());
    $hasRole = $currentUser->hasRole($rid);
    if ($hasRole) {
      $currentUser->removeRole($rid);
    }
    else {
      $currentUser->addRole($rid);
    }
    $currentUser->save();
    return !$hasRole;
  }

}
